@@ -2,7 +2,7 @@ from copy import deepcopy
 
 from gi.repository import Gtk
 
-from ...core import DataType, ObjectType, AccessType, StorageLocation, str2int
+from ...core import DataType, ObjectType, AccessType, str2int
 from ...core.eds import EDS
 from ...core.objects import Variable, Array, Record
 from ..dialogs.errors_dialog import ErrorsDialog
@@ -44,17 +44,17 @@ class ObjectDictionaryPage(Page):
         button.connect('clicked', self.on_expand_clicked)
         box_search.append(button)
 
-        self._od_scrolled_window = Gtk.ScrolledWindow()
-        self._od_scrolled_window.set_vexpand(True)
-        self._od_scrolled_window.set_hexpand(True)
-        self._od_scrolled_window.set_has_frame(True)
-        box_tree.append(self._od_scrolled_window)
+        scrolled_window = Gtk.ScrolledWindow()
+        scrolled_window.set_vexpand(True)
+        scrolled_window.set_hexpand(True)
+        scrolled_window.set_has_frame(True)
+        box_tree.append(scrolled_window)
 
         self._indexes_store = Gtk.TreeStore(str, str)
         self._tree_filter = self._indexes_store.filter_new()
         self._tree_filter.set_visible_func(self.tree_filter_func)
         self._od_treeview = Gtk.TreeView()
-        self._od_scrolled_window.set_child(self._od_treeview)
+        scrolled_window.set_child(self._od_treeview)
         self._od_treeview.set_model(self._tree_filter)
         self._od_treeview.set_enable_tree_lines(True)
 
@@ -94,13 +94,13 @@ class ObjectDictionaryPage(Page):
         grid = Gtk.Grid(column_spacing=5, row_spacing=5,
                         column_homogeneous=True, row_homogeneous=True,
                         margin_top=5, margin_bottom=5, margin_start=5, margin_end=5)
-        frame.set_valign(Gtk.Align.START)
         frame.set_child(grid)
 
         label = Gtk.Label.new('Parameter Name:')
         label.set_halign(Gtk.Align.START)
         self._obj_parameter_name = Gtk.Entry()
         self._obj_parameter_name.set_max_length(241)
+        self._obj_parameter_name.connect('changed', self._on_parametere_name_changed)
         grid.attach(label, column=0, row=0, width=1, height=1)
         grid.attach(self._obj_parameter_name, column=1, row=0, width=3, height=1)
 
@@ -108,6 +108,7 @@ class ObjectDictionaryPage(Page):
         label.set_halign(Gtk.Align.START)
         self._obj_denotation = Gtk.Entry()
         self._obj_denotation.set_max_length(241)
+        self._obj_denotation.connect('changed', self._on_obj_denotation_changed)
         grid.attach(label, column=0, row=1, width=1, height=1)
         grid.attach(self._obj_denotation, column=1, row=1, width=3, height=1)
 
@@ -127,6 +128,7 @@ class ObjectDictionaryPage(Page):
         access_type_list = Gtk.StringList.new(strings=[i.name for i in AccessType])
         self._obj_access_type.set_model(access_type_list)
         self._obj_access_type.set_selected(0)
+        self._obj_access_type.connect('activate', self._on_obj_access_type_changed)
         grid.attach(label, column=2, row=2, width=1, height=1)
         grid.attach(self._obj_access_type, column=3, row=2, width=1, height=1)
 
@@ -136,8 +138,10 @@ class ObjectDictionaryPage(Page):
         scrolled_window.set_vexpand(True)
         scrolled_window.set_hexpand(True)
         scrolled_window.set_has_frame(True)
-        self._obj_comment = Gtk.TextView()
-        scrolled_window.set_child(self._obj_comment)
+        text_view = Gtk.TextView()
+        self._obj_comment = text_view.get_buffer()
+        self._obj_comment.connect('changed', self._on_obj_comment_changed)
+        scrolled_window.set_child(text_view)
         grid.attach(label, column=0, row=3, width=1, height=5)
         grid.attach(scrolled_window, column=1, row=3, width=3, height=5)
 
@@ -147,6 +151,7 @@ class ObjectDictionaryPage(Page):
         data_type_list = Gtk.StringList.new(strings=[i.name for i in DataType])
         self._obj_data_type.set_model(data_type_list)
         self._obj_data_type.set_selected(0)
+        self._obj_data_type.connect('activate', self._on_obj_data_type_changed)
         grid.attach(label, column=0, row=8, width=1, height=1)
         grid.attach(self._obj_data_type, column=1, row=8, width=1, height=1)
 
@@ -155,74 +160,101 @@ class ObjectDictionaryPage(Page):
         self._obj_pdo_mapping = Gtk.Switch()
         self._obj_pdo_mapping.set_halign(Gtk.Align.START)
         self._obj_pdo_mapping.set_valign(Gtk.Align.CENTER)
+        self._obj_pdo_mapping.connect('activate', self._on_obj_pdo_mapping_changed)
         grid.attach(label, column=2, row=8, width=1, height=1)
         grid.attach(self._obj_pdo_mapping, column=3, row=8, width=1, height=1)
 
         label = Gtk.Label.new('Default Value:')
         label.set_halign(Gtk.Align.START)
         self._obj_default_value = Gtk.Entry()
+        self._obj_default_value.connect('changed', self._on_obj_default_value_changed)
         grid.attach(label, column=0, row=9, width=1, height=1)
         grid.attach(self._obj_default_value, column=1, row=9, width=3, height=1)
 
         label = Gtk.Label.new('Low Limit:')
         label.set_halign(Gtk.Align.START)
         self._obj_low_limit = Gtk.Entry()
+        self._obj_low_limit.connect('changed', self._on_obj_low_limit_changed)
         grid.attach(label, column=0, row=10, width=1, height=1)
         grid.attach(self._obj_low_limit, column=1, row=10, width=1, height=1)
 
         label = Gtk.Label.new('High Limit:')
         label.set_halign(Gtk.Align.START)
         self._obj_high_limit = Gtk.Entry()
+        self._obj_high_limit.connect('changed', self._on_obj_high_limit_changed)
         grid.attach(label, column=2, row=10, width=1, height=1)
         grid.attach(self._obj_high_limit, column=3, row=10, width=1, height=1)
 
         label = Gtk.Label.new('Storage Location (CANopenNode):')
         label.set_halign(Gtk.Align.START)
         self._obj_storage_loc = Gtk.DropDown()
-        storage_loc_list = Gtk.StringList.new(strings=[i.name for i in StorageLocation])
-        self._obj_storage_loc.set_model(storage_loc_list)
-        self._obj_storage_loc.set_selected(0)
+        if eds.storage_locations:
+            self._obj_storage_loc.set_model(Gtk.StringList.new(strings=eds.storage_locations))
+            self._obj_storage_loc.set_selected(0)
+            self._obj_storage_loc.connect('activate', self._on_obj_storage_loc_changed)
         grid.attach(label, column=0, row=11, width=1, height=1)
         grid.attach(self._obj_storage_loc, column=1, row=11, width=1, height=1)
 
-        button = Gtk.Button(label='Update')
-        button.set_halign(Gtk.Align.END)
-        button.set_valign(Gtk.Align.END)
-        button.connect('clicked', self.on_update_button_clicked)
-        grid.attach(button, column=0, row=12, width=2, height=2)
+        self.refresh()
 
-        button = Gtk.Button(label='Cancel')
-        button.set_halign(Gtk.Align.START)
-        button.set_valign(Gtk.Align.END)
-        button.connect('clicked', self.on_cancel_button_clicked)
-        grid.attach(button, column=2, row=12, width=2, height=2)
+    def _on_parametere_name_changed(self, entry: Gtk.Entry):
+        if self._selected_obj and self._selected_obj.parameter_name != entry.get_text():
+            self._selected_obj.parameter_name = entry.get_text()
+            self.eds_changed()
 
-        self.reload_eds()
+    def _on_obj_denotation_changed(self, entry: Gtk.Entry):
+        if self._selected_obj and self._selected_obj.denotation != entry.get_text():
+            self._selected_obj.denotation = entry.get_text()
+            self.eds_changed()
 
-    def on_update_button_clicked(self, button: Gtk.Button):
-        '''Update button callback to save changes the selected object.'''
+    def _on_obj_access_type_changed(self, dropdown: Gtk.DropDown):
+        access_type = AccessType[dropdown.get_value()]
+        if self._selected_obj and self._selected_obj.access_type != access_type:
+            self._selected_obj.access_type = access_type
+            self.eds_changed()
 
-        if self._selected_obj is None:
-            return
+    def _on_obj_comment_changed(self, buffer: Gtk.TextBuffer):
+        text = buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter(), False)
+        if self._selected_obj and self._selected_obj.comments != text:
+            self._selected_obj.comments = text
+            self.eds_changed()
 
-        self._eds_changed = True
+    def _on_obj_data_type_changed(self, dropdown: Gtk.DropDown):
+        data_type = DataType[dropdown.get_value()]
+        if self._selected_obj and self._selected_obj.data_type != data_type:
+            self._selected_obj.data_type = data_type
+            self.eds_changed()
 
-        self._selected_obj.parameter_name = self._obj_parameter_name.get_text()
-        buf = self._obj_comment.get_buffer()
-        self._selected_obj.comments = buf.get_text(buf.get_start_iter(), buf.get_end_iter(), False)
-        data_type = self._obj_data_type.get_selected()
-        self._selected_obj.data_type = list(DataType)[data_type]
-        access_type = self._obj_access_type.get_selected()
-        self._selected_obj.access_type = list(AccessType)[access_type]
-        self._selected_obj.default_value = self._obj_default_value.get_text()
-        self._selected_obj.pdo_mapping = self._obj_pdo_mapping.get_state()
-        storage_loc = self._obj_storage_loc.get_selected()
-        self._selected_obj.storage_location = list(StorageLocation)[storage_loc]
+    def _on_obj_pdo_mapping_changed(self, switch: Gtk.Switch):
+        state = switch.get_state()
+        if self._selected_obj and self._selected_obj.pdo_mapping != state:
+            self._selected_obj.pdo_mapping = state
+            self.eds_changed()
 
-    def on_cancel_button_clicked(self, button: Gtk.Button):
-        '''Cancel button callback to cancel changes the selected object.'''
+    def _on_obj_default_value_changed(self, entry: Gtk.Entry):
+        text = entry.get_text()
+        if self._selected_obj and self._selected_obj.default_value != text:
+            self._selected_obj.default_value = text
+            self.eds_changed()
 
-        self._load_selection()
+    def _on_obj_low_limit_changed(self, entry: Gtk.Entry):
+        text = entry.get_text()
+        if self._selected_obj and self._selected_obj.low_limit != text:
+            self._selected_obj.low_limit = text
+            self.eds_changed()
+
+    def _on_obj_high_limit_changed(self, entry: Gtk.Entry):
+        text = entry.get_text()
+        if self._selected_obj and self._selected_obj.high_limit != text:
+            self._selected_obj.high_limit = text
+            self.eds_changed()
+
+    def _on_obj_storage_loc_changed(self, dropdown: Gtk.DropDown):
+        if self._eds.storage_locations:
+            storage_location = self._eds.storage_locations[dropdown.get_value()]
+            if self._selected_obj and self._selected_obj.storage_location != storage_location:
+                self._selected_obj.storage_location = storage_location
+                self.eds_changed()
 
     def on_search_entry(self, entry):
         '''Callback on search filter entry for parameter names'''
@@ -232,7 +264,7 @@ class ObjectDictionaryPage(Page):
             self._tree_filter.refilter()
 
     def tree_filter_func(self, model, iter, data) -> bool:
-        '''Callback on refilter to change row visibility
+        '''Callback on refilter to changed row visibility
 
         Return
         ------
@@ -271,9 +303,10 @@ class ObjectDictionaryPage(Page):
         self._obj_denotation.set_text(self._selected_obj.denotation)
         obj_type = self._selected_obj.object_type
         self._obj_type.set_selected(list(ObjectType).index(obj_type))
-        self._obj_comment.get_buffer().set_text(self._selected_obj.comments)
+        self._obj_comment.set_text(self._selected_obj.comments)
         storage_loc = self._selected_obj.storage_location
-        self._obj_storage_loc.set_selected(list(StorageLocation).index(storage_loc))
+        if storage_loc:
+            self._obj_storage_loc.set_selected(self._eds.storage_locations.index(storage_loc))
         if self._selected_obj.object_type == ObjectType.VAR:
             data_type = self._selected_obj.data_type
             self._obj_data_type.set_selected(list(DataType).index(data_type))
@@ -291,7 +324,7 @@ class ObjectDictionaryPage(Page):
                 self._obj_data_type.set_selected(0)
 
     def on_tree_selection_changed(self, selection: Gtk.SelectionModel):
-        '''When value is selected in the treeview, change the current selected object.'''
+        '''When value is selected in the treeview, changed the current selected object.'''
         model, treeiter = selection.get_selected()
 
         if not treeiter:
@@ -348,8 +381,9 @@ class ObjectDictionaryPage(Page):
 
         self._load_selection()
 
-    def reload_eds(self):
-        '''Reload the eds in the treeview.'''
+    def refresh(self):
+        '''Refresh the page'''
+
         self._indexes_store.clear()
 
         # fill tree view with object dictionary data
@@ -378,7 +412,7 @@ class ObjectDictionaryPage(Page):
             The name of the new object.
         '''
 
-        self._eds_changed = True
+        self.eds_changed()
         index_str = f'0x{index:X}'
 
         index_found = False
@@ -434,7 +468,7 @@ class ObjectDictionaryPage(Page):
             The subindex of the object to remove.
         '''
 
-        self._eds_changed = True
+        self.eds_changed()
 
         for i in self._indexes_store:
             if index == str2int(i[0]):
@@ -462,7 +496,7 @@ class ObjectDictionaryPage(Page):
             The name of the new object.
         '''
 
-        self._eds_changed = True
+        self.eds_changed()
 
         for i in self._indexes_store:
             if index == str2int(i[0]):
@@ -484,9 +518,9 @@ class ObjectDictionaryPage(Page):
         dialog.show()
 
     def add_treeview_object_response(self, dialog: Gtk.Dialog, response: int):
-        '''Parses the response to the ad object dialog..'''
+        '''Parses the response to the add object dialog.'''
 
-        self._eds_changed = True
+        self.eds_changed()
 
         new_index, new_subindex, new_obj_type = dialog.get_response()
 
@@ -548,7 +582,7 @@ class ObjectDictionaryPage(Page):
         if self._selected_index not in self._eds.indexes:
             return  # nothing to delete
 
-        self._eds_changed = True
+        self.eds_changed()
 
         # remove from od
         if self._selected_subindex is None:
@@ -570,7 +604,7 @@ class ObjectDictionaryPage(Page):
     def copy_treeview_object_response(self, dialog: Gtk.Dialog, response: int):
         '''Parses the response to the copy object dialog..'''
 
-        self._eds_changed = True
+        self.eds_changed()
 
         new_index, new_subindex = dialog.get_response()
 
@@ -608,7 +642,7 @@ class ObjectDictionaryPage(Page):
     def move_treeview_object_response(self, dialog: Gtk.Dialog, response: int):
         '''Parses the response to the move object dialog..'''
 
-        self._eds_changed = True
+        self.eds_changed()
 
         new_index, new_subindex = dialog.get_response()
 
