@@ -19,11 +19,22 @@ class Page(Gtk.ScrolledWindow):
         self._eds_changed = False
 
     @property
-    def eds_changed(self) -> bool:
+    def eds_has_changed(self) -> bool:
         '''bool: a flag to see if eds info has changed.'''
+
         return self._eds_changed
 
+    def eds_changed(self):
+        '''Set the eds_changed flag'''
+
+        self._eds_changed = True
+
     def eds_changed_reset(self):
         '''Reset the eds changed flag.'''
 
         self._eds_changed = False
+
+    def refresh(self):
+        '''Refresh the page'''
+
+        pass
